@ rem -  Library:       WebSphere MQ
@ rem -  Component:     WMQ SOAP Sample Programs
@ rem -  Part:          SecondAxis.java
@ rem - 
@ rem -   <START_COPYRIGHT> 
@ rem -   Licensed Materials - Property of IBM 
@ rem -                    
@ rem -   5724-B41         
@ rem -   (C) Copyright IBM Corp. 1994, 2003 All Rights Reserved. 
@ rem -                    
@ rem -   US Government Users Restricted Rights - Use, duplication or 
@ rem -   disclosure restricted by GSA ADP Schedule Contract with 
@ rem -   IBM Corp.        
@ rem -   <END_COPYRIGHT>
@ rem - setup classpath and path as required for WMQSoap
@ rem - make sure we are working in our own directory
@ rem - and get MA0R_HOME as an absolute paths usbale from anywhere
@ rem - but without the .. noise
@echo off

rem - this prepares a JAVA file for access by SOAP/WMQ
rem - see help (at end, called by deployWMQService with no arguments) for details.

setlocal

call :setup %*
if errorlevel 1 (
   endlocal
   exit /b 1
)

call :%todo%
if errorlevel 1 (
   endlocal
   exit /b 10
)

endlocal
exit /b 0
rem ===========================================================

:setup
rem - standard classpath etc for all MQSOAP stuff
call %~dp0\setcp
if errorlevel 1 exit /b 1

rem - set default queue manager unless overridden
set mqqm=""
set mqq=#default#
set todo=default

:moreparms
if "%1"=="" if not "%todo:~0,5%"=="start" goto :errhelp
if "%1"=="?" goto :help
if "%1"=="-?" goto :help

rem - pick up queue manager if defined
if %1==-m (
   set mqqm=%2
   shift
   shift
   goto :moreparms
)

rem -pick up queue if defined
if %1==-q (
   set mqq=%2
   shift
   shift
   goto :moreparms
)

rem - pick up command todo
if %1==-c (
   set todo=%2
   shift
   shift
   goto :moreparms
)

rem - pick up command todo
if %1==-echo (
   echo  %2
   shift
   shift
   goto :moreparms
)

rem - add .NET Framework SDK to the path as needed.  Needed to generate the .NET proxies.
if "%msfwdir%"=="" set msfwdir=C:\Program Files\Microsoft Visual Studio .NET\FrameworkSDK\Bin
call :whereInPath wsdl.exe
if "%whereInPath%"=="" set path=%msfwdir%;%path%

rem - double check this one, and correct if appropriate,
rem - warning. don't use nested if code to optimize this, batch processor gets it wrong
call :whereInPath wsdl.exe
if "%whereInPath%"=="" (
  echo wsdl.exe NOT FOUND +++++++++++++++++++++++++++++++++++++++++++++
  echo Make sure the .NET framework is in the path, 
  echo or correct %0 to identify where the SDK is installed.  see "set msfwdir=..."
  exit /b 1
)  


rem ------------------
set className=%1

rem - remove the .java if specified, and make sure in class (.) not directory (\) format
if "%className:~-5%"==".java" (
  set className=%className:~0,-5%
  if %todo%==default set todo=allAxis
)  

if "%className:~-5%"==".asmx" (
  set className=%className:~0,-5%
  if %todo%==default set todo=allAsmx
)  

set className=%className:\=.%

set classSourceDir=%className:.=\%

rem - equal if no package used, in which case Axis will generate package DefaultNamespace for proxies
set proxyDir=%classSourceDir%
if %classSourceDir%==%className% set proxyDir=DefaultNamespace\%className%
set proxyName=%proxyDir:\=.%

rem - prepare some useful derived names
set wmq_wsdl=%className%_Wmq.wsdl
if %mqq%==#default# set mqq=SOAP.%className%
if %mqqm%=="" (
   set wmqUri=wmq:%mqq%
) else (
   set wmqUri=wmq:%mqq%@%mqqm%?connectQueueManager=%mqqm%
)
set servicePort=%className%_Wmq

rem - create work directories (if needed)
md helpers 2>/nul

exit /b 0
rem --------------- end setup
rem ======================================================================================

:allAxis
   call :compileJava
                    if errorlevel 1 exit /b %errorlevel%
   call :genAxisWsdl
                    if errorlevel 1 exit /b %errorlevel%
   call :axisDeploy
                    if errorlevel 1 exit /b %errorlevel%
   call :genAxisWMQBits
                    if errorlevel 1 exit /b %errorlevel%
   call :genProxiesToAxis
                    if errorlevel 1 exit /b %errorlevel%
exit /b 0
rem --------------- end allAxis
rem ======================================================================================

:allAsmx
   call :genAsmxWsdl
                    if errorlevel 1 exit /b %errorlevel%
   call :genAsmxWMQBits
                    if errorlevel 1 exit /b %errorlevel%
   call :genProxiesToDotNet
                    if errorlevel 1 exit /b %errorlevel%
exit /b 0
rem --------------- end allAxis
rem ======================================================================================


:compileJava
rem - compile the java, create target directory if necessary
md classes 2>/nul
javac -d classes %classSourceDir%.java
if %errorlevel% neq 0 (
   @echo Failure in compilation of %proxyDir%.java
   exit /b %errorlevel%
)
exit /b 0
rem ------------ end compile
rem ======================================================================================

:genAxisWsdl
rem - generate the correct wsdl, do not override any old versions
if exist helpers\%wmq_wsdl% ( set wsdlin=--input helpers\%wmq_wsdl% ) else set wsdlin=
java org.apache.axis.wsdl.Java2WSDL %wsdlin% --output helpers\%wmq_wsdl% --location "%wmqUri%" --bindingName %className%BindingSoap --servicePortName %servicePort% %className%
if not exist helpers\%wmq_wsdl% (
   echo Failure in building wsdl
   exit /b 5
)
exit /b 0
rem --------------- end genAxisWsdl
rem ======================================================================================

:genAsmxWsdl
%~dp0\mqsoapwsdl %wmqUri% %className%.asmx helpers\%WMQ_WSDL%
if not exist helpers\%wmq_wsdl% (
   echo Failure in building Wsdl
   exit /b 5
)
if errorlevel 1 (
   echo Error in building Wsdl, see 'helpers\%wmq_wsdl%' for details
   exit /b %errorlevel%
)
exit /b 0
rem --------------- end genAxisWsdl
rem ======================================================================================

:genAxisWMQBits
rem - prepare a tailored listener with helper .bat file (and queues to match)
set listen=java com.ibm.axis.transport.wmq.SimpleJMSListener -u "%wmqUri%" -s %servicePort%
set listenbat=helpers\listen_%className%.bat  
@echo rem - generated by %0  on %date% at %time% >%listenbat%
@echo call %~dp0\setcp >>%listenbat%
@echo cd /d %cd% >>%listenbat%
@echo %listen% >>%listenbat%

rem - define queue for MQ, and set it up to trigger its listener
call :runMQCommand DEFINE QL('SOAP.INITQ') defsopt(excl) REPLACE
call :runMQCommand DEFINE QL('%mqq%') TRIGTYPE(FIRST) TRIGGER PROCESS('SOAP.%className%') INITQ('SOAP.INITQ') TRIGMPRI(0) REPLACE
call :runMQCommand DEFINE PROCESS('SOAP.%className%') APPLICID('start "JMS WMQSoapListener - %className%" /min %cd%\%listenbat%') REPLACE
exit /b 0
rem --------------- end genAxisWMQBits
rem - Comment on directories.  We need to be very careful with directories here.
rem - There are several directories of interst:
rem -    1) The directory in which the service 'lives'.
rem -         (e.g. c:/ma0r/demos)
rem -         (e.g. e:/myservice)
rem -       This will also be %cd% directory while this utility is run at deploy time,
rem -       and the directory in which the listener must finally run.
rem -       This will permit 
rem -          Axis to find the correct server-config.wsdd,
rem -          the service class to be picked up (from classes/<package>/<service>.class
rem -          the service to find any configurtion files etc it may need
rem -    2) the directory in which the generated .bat file lives to start the listener
rem -         (e.g. c:/ma0r/demos/helpers)
rem -         (e.g. e:/myservice/helpers)
rem -    3) the current directory when the generated listenter_xxx.bat is invoked
rem -         (e.g. c:/ma0r/demos)
rem -         (e.g. e:/rubbish)
rem -         This could be the trigger monitor directory (when called by triggering)
rem -         or some arbitrary user directory.
rem -         The listener_xxx.bat file must operate correctly whatever this directory.
rem -         Hence the 'cd /d %cd' line, which will force execution in the service directory.
rem -         This is expanded at deploy time into a fully qualified path in listenter_xxx.bat,
rem -         thus forcing the listener into the correct service directory at execution time.
rem -    4) the service source directory.  This is assumed to be based on the service execution directory.
rem -         (e.g. c:/ma0r/demos/javaDemos/server)
rem -         (e.g. e:/myservice/package)
rem -    5) the service class directory.  
rem -          This is based on the class subdirectory of the service execution directory.
rem -         (e.g. c:/ma0r/demos/classes/javaDemos/server)
rem -         (e.g. e:/myservice/classes/package)
rem -         Hence the need for classes as a relative directory in classpath.  (see setcp)
rem -    6) the directory where ma0r executables live
rem -         (e.g. c:/ma0r/demos/bin)
rem -         This is the same directory as this utility deployWMQService.bat,
rem -         and is 'discovered' by %~dp0
rem -    7) the directory where ma0r .jar files live
rem -         (e.g. c:/ma0r/demos/lib)
rem -         This is 'discovered' by setcp and saved in the classpath.
rem -   
rem -
rem ======================================================================================


:genAsmxWMQBits
rem - prepare a tailored listener with helper .bat file (and queues to match)
set listen=%~dp0\MQSoapHost  %mqqm% %mqq% %cd% /vdir %className%.asmx
set listenbat=helpers\listen_%className%.bat  
@echo rem - generated by %0  on %date% at %time% >%listenbat%
@echo call %~dp0\setcp >>%listenbat%
@echo %listen% >>%listenbat%

rem - define queue for MQ, and set it up to trigger its listener
call :runMQCommand DEFINE QL('SOAP.INITQ') REPLACE
call :runMQCommand DEFINE QL('%mqq%') TRIGTYPE(FIRST) TRIGGER PROCESS('%className%.PROCESS') INITQ('SOAP.INITQ') TRIGMPRI(0) REPLACE
call :runMQCommand DEFINE PROCESS('%className%.PROCESS') APPLICID('start "WMQAsmxListener - %className%" /min /d%cd% %listenbat%') REPLACE
exit /b 0
rem --------------- end genAxisWMQBits
rem ======================================================================================


:startWMQMonitor
start "MQ SOAP trigger monitor" /min runmqtrm -m %mqqm% -q SOAP.INITQ
exit /b
rem --------------- end genAxisWMQBits
rem ======================================================================================


:genProxiesToAxis
rem - Generate all proxies (C#, VB and Java) and server wsdd file, and compile the Java
pushd helpers
call :quietOK wsdl %WMQ_WSDL% 
call :quietOK wsdl %WMQ_WSDL% /language:VB

java com.ibm.mq.ma0r.tools.RunWSDL2Java %WMQ_WSDL%
md classes 2>/nul

call :getDir %proxyDir%
javac -d classes %tempDir%*.java
popd
exit /b 0
rem --------------- end genProxies
rem ======================================================================================

:genProxiesToDotNet
rem - Generate all proxies (C#, VB and Java) and server wsdd file, and compile the Java
pushd helpers
call :quietOK wsdl %WMQ_WSDL% 
call :quietOK wsdl %WMQ_WSDL% /language:VB

rem - Not sure of full convention for .NET generation of namespaces
rem - and subsequent picking up into packages by Axis.
rem - This needs more looking into.
rem  -Meanwhile, force all proxies into the dotNetService package.
java com.ibm.mq.ma0r.tools.RunWSDL2Java -p dotNetService %WMQ_WSDL%
md classes 2>/nul
javac -d classes dotNetService\%className%*.java
popd
exit /b 0
rem --------------- end genProxies
rem ======================================================================================



:axisDeploy
rem - Generate the server side wsdd file
rem - There is not an option in to avoid generating the client stuff
rem - so we work in a temporary directory, copy out what we want, and then erase the directory.
rem -
rem - Where the client stuff is required as well, calling twice for the proxies and the wsdd is waseful
rem - but improves the code modularity.
rem -
rem - java com.ibm.mq.ma0r.tools.RunWSDL2Java is the same as org.apache.axis.wsdl.WSDL2Java,
rem - except that it allows for the wmq: URL syntax.
pushd helpers
call :quietOK java com.ibm.mq.ma0r.tools.RunWSDL2Java --server-side --output temp.server %WMQ_WSDL%
rem - patch the wsdd -- Axis does not seem to understand that the wsdl was generated bottom up.
rem - This very primitive substituation points the wsdd directly at our original class as implementation,
rem - rather than at the newly generated implementation skeleton.
rem -
rem - make sure target directory ready
call :getDir %classSourceDir%
md %tempDir% 2>/nul

rem - find the temporary source directory
call :getDir temp.server\%proxyDir%

rem - now do the patching
java com.ibm.mq.ma0r.util.PatchWsdd %className% <%tempDir%deploy.wsdd >%classSourceDir%_deploy.wsdd
java com.ibm.mq.ma0r.util.PatchWsdd %className% <%tempDir%undeploy.wsdd >%classSourceDir%_undeploy.wsdd

erase /q temp.server
popd

rem - deploy using the generated (and modified) wsdd file
java org.apache.axis.utils.Admin server helpers\%classSourceDir%_deploy.wsdd

exit /b 0
rem --------------- end axisDeploy
rem ======================================================================================

:default
@echo No command (-c) specified for deployWMQService.
@echo A command must be specified unless the input is specified as a file with the .java or .asmx extension.
@echo Type deployWMQService -? for more details.
exit /b 1
rem --------------- end axisDeploy
rem ======================================================================================


rem ======================================================================================
rem - helper functions
rem - given filename with path, extract just the directory, and set into variable tempDir
:getDir
set tempDir=%~p1
exit /b 0

rem - execute a command quietly (unless it fails)
:quietOK
%* >quietOK.templog  2>quietOK.temperrlog
set quieOKExit=%errorlevel%
if not %errorlevel% == 0 (
   echo +++++++++++++++ quietOK %* gave errlevel = %errorlevel%
   type quietOK.templog
   type quietOK.temperrlog
)
erase quietOK.templog
erase quietOK.temperrlog
exit /b %quietOKExit%


rem ======================================================================================
:runMQCommand
rem - execute a WMQ command, quitely if it works
echo %* | runmqsc %mqqm%  >quietOK.templog  2>quietOK.temperrlog
set quieOKExit=%errorlevel%
if not %errorlevel% == 0 (
   echo +++++++++++++++ runmqsc gave errlevel = %errorlevel%
   type quietOK.templog
   type quietOK.temperrlog
)
erase quietOK.templog
erase quietOK.temperrlog
exit /b %quietOKExit%

rem ======================================================================================
rem - help
:errhelp
@echo Incorrect parameters given to %0 +++++++++++++++++++++++++++++++++++++++++++++++++++++++
@echo -

:help
@echo %0 [options] className:        prepare classname for access by SOAP/WMQ
@echo -
@echo arguments:
@echo     className:       name of Java class or file to be deployed (.java optional).
@echo                        java file must be in the conventional package dependent subdirectory
@echo options:
@echo     -echo ON/OFF     set trace on or off for execution of this file
@echo     -m qmgrName      queue manager to use (default is the default queue manager)
@echo     -q queueName     queue to use (default is "SOAP.<className>")
@echo     -c command       command to execute, where command is:
@echo         allAxis         peform all compile and setup steps for an Axis service
@echo            compileJava         compile the Java service (.java   to .class)
@echo            genAxisWsdl         generate Wsdl            (.class  to .wsdl)
@echo            axisDeploy          deploy the class file    (.wsdl   to .wsdd, apply .wsdd)
@echo            genProxiestoAxis    generate proxies         (.wsdl   to .java, .class, .cs and .vb)
@echo            genAxisWMQBits      setup WMQ queues, listeners and triggers
@echo -
@echo         allAsmx         peform all setup steps for an Asmx service
@echo            genAsmxWsdl         generate Wsdl            (.asmx   to .wsdl)
@echo            genProxiesToDotNet  generate proxies         (.wsdl   to .java, .class, .cs and .vb)
@echo            genAsmxWMQBits      setup WMQ queues, listeners and triggers
@echo -
@echo         startWMQMonitor start the trigger monitor for WMQSOAP services.
@echo -
@echo outputs:
@echo     The source file is compiled into ".\classes"
@echo     Other files are created in ".\helpers",
@echo       and various package dependent ".\helpers\<package>" subdirectories.
@echo       Java proxies are compiled into ".\helpers.classes".
@echo -

exit /b 1


@ rem - subroutine to check if item already in path (and if so, where)
@ :whereInPath
@ set whereInPath=%~dp$PATH:1
@ exit /b 0

